local addon, ns = ...

local f = ns.f
local db = ns.db
local profile = ns.profile
local L = ns.L

f.bars = {}

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:AddBar()
    local barID
    for k, v in f:pairsByKeys(profile.removedBars, function(a, b) return a < b end) do
        barID = k
        break
    end
    if barID then
        profile.removedBars[barID] = nil
        profile.bars[barID] = f:NewBar()
        f:LoadBar(barID)
        return barID
    else
        tinsert(profile.bars, f:NewBar())
        f:LoadBar(f:count(profile.bars))
        return f:count(profile.bars)
    end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:DestroyBar(barID)
    local barFrame = f.bars[barID]
    barFrame:ClearAllPoints()
    barFrame:Hide()

    profile.bars[barID] = nil
    f.bars[barID] = nil

    profile.removedBars[barID] = true
    f:Refresh()
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:LoadBar(barID)
    local style = f:GetStyle(db.settings.style)
    if profile.bars[barID] then
        
        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Bar frame

        local barFrame = f.bars[barID] or CreateFrame("Frame", addon .. "BarFrame" .. barID, UIParent)
        barFrame.db = profile.bars[barID]
        barFrame.barID = barID

        f:SetFrameTexture(barFrame, style.bar)
        barFrame:SetSize(35, 35)
        barFrame:SetScale(barFrame.db.scale)
        barFrame:SetPoint(f:unpack(barFrame.db.point))

        f.bars[barID] = barFrame
        
        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Click handler

        barFrame:SetFrameStrata("HIGH")
        barFrame:EnableMouse(true)

        barFrame:SetScript("OnMouseUp", function(self, button, ...)
            if button == "LeftButton" then
                if IsShiftKeyDown() then
                    if barFrame:IsMovable() then
                        f:TransformBar("movable", false, "SetMovable", barID, L["%s: position locked."])
                    else
                        f:TransformBar("movable", true, "SetMovable", barID, L["%s: position unlocked."])
                    end

                    f:Refresh()
                end
            elseif button == "RightButton" then
                if IsShiftKeyDown() then
                    f:SetConfigID(barID)
                    f:LoadTab("Config")
                else
                    f:LoadTab("Help")
                end
            end
        end)

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Drag handler

        barFrame:SetMovable(barFrame.db.movable)
        barFrame:SetClampedToScreen(true)
        barFrame:RegisterForDrag("LeftButton")

        barFrame:SetScript("OnDragStart", function(self) if self:IsMovable() then self:StartMoving() end end)
        barFrame:SetScript("OnDragStart", function(self) if self:IsMovable() then self:StartMoving() end end)
        barFrame:SetScript("OnDragStop", function(self) if self:IsMovable() then self:StopMovingOrSizing(); barFrame.db.point = {self:GetPoint()} end end)
        
        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Tooltip

        barFrame:SetScript("OnEnter", function(self, ...)
            if db.settings.tooltip.bar then
                GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 0, 0)
                GameTooltip:AddLine(string.format(L["%s Bar #%d"], addon, barID), 0, 1, 0)
                GameTooltip:AddLine(" ")
                GameTooltip:AddLine(string.format(L["Maximum Slots Available: %d"], f.maxButtons), 1, .82, 0)
                GameTooltip:AddLine(string.format(L["Slots Visible: %d"], barFrame.db.numButtons), 1, .82, 0)
                GameTooltip:AddLine(string.format(L["Grow: %s"], barFrame.db.orientation), 1, .82, 0)
                GameTooltip:AddLine(string.format(L["Scale: %d%%"], (barFrame.db.scale * 100)), 1, .82, 0)
                GameTooltip:AddLine(string.format(L["Alpha: %d%%"], (barFrame.db.alpha * 100)), 1, .82, 0)
                GameTooltip:AddLine(" ")
                GameTooltip:AddLine(string.format(L["Position Locked: %s"], barFrame:IsMovable() and "FALSE" or "TRUE"), 1, .82, 0)
                GameTooltip:AddLine(" ")
                GameTooltip:AddLine("------------------------------------------------", 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffClick|r and drag to move the bar."], 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffShift+Click|r to lock bar position."], 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffRight-Click|r opens the help page."], 1, 1, 1)
                GameTooltip:AddLine(L["|cff00ccffShift+Right-Click|r for config options."], 1, 1, 1)
                GameTooltip:Show()
            end
        end)

        barFrame:SetScript("OnLeave", function()
            GameTooltip:Hide()
        end)

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Remove/add buttons

        local removeButton = barFrame.buttons and barFrame.buttons.removeButton or CreateFrame("Button", "$parentRemoveButton", barFrame)
        removeButton:SetSize(15, 15)
        removeButton:SetPoint("TOP", -(removeButton:GetWidth() / 2), -2)

        removeButton:SetNormalTexture("Interface\\BUTTONS\\UI-MinusButton-Up")
        removeButton:SetHighlightTexture("Interface\\BUTTONS\\UI-ScrollBar-ScrollDownButton-Highlight")
        removeButton:SetPushedTexture("Interface\\BUTTONS\\UI-MinusButton-Down")
        removeButton:SetDisabledTexture("Interface\\BUTTONS\\UI-MinusButton-Disabled")

        removeButton:SetScript("OnClick", function(self)
            if UnitAffectingCombat("player") then
                self:RegisterEvent("PLAYER_REGEN_ENABLED")
                f:print("Cannot adjust bars during combat. Will resume after combat ends.")
            else
                barFrame.db.numButtons = barFrame.db.numButtons > 0 and (barFrame.db.numButtons - 1) or barFrame.db.numButtons
                f:LoadButtons(barFrame)

                f:UpdateBarFrame(barID)
            end
        end)

        removeButton:SetScript("OnEvent", function(self, event, ...)
            if event == "PLAYER_REGEN_ENABLED" then
                self:Click()
                self:UnregisterEvent(event)
            end
        end)

        local addButton = barFrame.buttons and barFrame.buttons.addButton or CreateFrame("Button", "$parentAddButton", barFrame)
        addButton:SetSize(15, 15)
        addButton:SetPoint("TOPLEFT", removeButton, "TOPRIGHT", 0, 0)

        addButton:SetNormalTexture("Interface\\BUTTONS\\UI-AttributeButton-Encourage-Up")
        addButton:SetHighlightTexture("Interface\\BUTTONS\\UI-AttributeButton-Encourage-Hilight")
        addButton:SetPushedTexture("Interface\\BUTTONS\\UI-AttributeButton-Encourage-Down")
        addButton:SetDisabledTexture("Interface\\BUTTONS\\UI-PlusButton-Disabled")

        addButton:SetScript("OnClick", function(self)
            if UnitAffectingCombat("player") then
                self:RegisterEvent("PLAYER_REGEN_ENABLED")
                f:print("Cannot adjust bars during combat. Will resume after combat ends.")
            else
                barFrame.db.numButtons = barFrame.db.numButtons < f.maxButtons and (barFrame.db.numButtons + 1) or barFrame.db.numButtons
                f:LoadButtons(barFrame)

                f:UpdateBarFrame(barID)
            end
        end)

        addButton:SetScript("OnEvent", function(self, event, ...)
            if event == "PLAYER_REGEN_ENABLED" then
                self:Click()
                self:UnregisterEvent(event)
            end
        end)

        barFrame.buttons = {
            ["addButton"] = addButton,
            ["removeButton"] = removeButton,
        }

        f:LoadButtons(barFrame)
        f:UpdateBarFrame(barID)
        
        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Bar number

        local barNumber = barFrame:CreateFontString(nil, "OVERLAY", style.fonts.normal)
        barNumber:SetPoint("BOTTOM", 0, 6)
        barNumber:SetText(barID)

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Bar visibility

        barFrame:SetAlpha(barFrame.db.alpha)
        if barFrame.db.hidden then
            barFrame:Hide()
        else
            barFrame:Show()
        end
    end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:NewBar()
    local bar = {
            orientation = "Up",
            numButtons = 4,
            items = {},
            scale = 1,
            alpha = 1,
            movable = true,
            hidden = false,
            tooltip = true,
            loadTemplate = false,
            point = {"CENTER", 0, 0},
        }

    return bar
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:SetBarSize(barID, value)
    profile.bars[barID].numButtons = value
    f:LoadButtons(f.bars[barID])
    f:UpdateBarFrame(barID)
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:TransformBar(dbKey, dbValue, transform, barID, msg)
    if barID and f.bars[tonumber(barID)] then
        barID = tonumber(barID)
        profile.bars[barID][dbKey] = dbValue
        f.bars[barID][transform](f.bars[barID], dbValue)
    elseif not barID then
        for k, v in pairs(profile.bars) do
            v[dbKey] = dbValue
            f.bars[k][transform](f.bars[k], dbValue)
        end
    else
        f:print("Invalid bar ID. Available bars: %s", f:GetValidString(profile.bars))
        return false
    end

    if msg then
        f:print(string.format(msg, barID and string.format(L["Bar %s"], barID) or L["All bars"]))
    end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:UpdateBarFrame(barID)
    local removeButton = _G[addon .. "BarFrame" .. barID .. "RemoveButton"]
    local addButton = _G[addon .. "BarFrame" .. barID .. "AddButton"]

    if profile.bars[barID].numButtons == 0 then
        removeButton:Disable()
        addButton:Enable()
    elseif profile.bars[barID].numButtons < f.maxButtons then
        removeButton:Enable()
        addButton:Enable()
    else
        addButton:Disable()
    end

    f:Refresh()
end