local addon, ns = ...

local f = ns.f
local db = ns.db
local profile = ns.profile
local L = ns.L

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

f.ValidArguments = {
    ["bar"] = {"add", "list", "remove"},
    ["tpl"] = {"delete", "list", "load", "save"},
    ["grow"] = {
        ["D"] = "Down",
        ["L"] = "Left",
        ["R"] = "Right",
        ["U"] = "Up",
    },
    ["style"] = {"default", "minimal"},
    ["tooltip"] = {"bar", "button"},
    ["rebuild"] = {"all", "global", "profile"},
    ["reset"] = {"all", "cache"},
}

f.ValidCommands = {
    ["alerts"] = function(arg)
        arg = arg and strlower(arg)
        if arg == "chat" or arg == "screen" or arg == "sound" then
            if db.settings.alerts[arg] then
                db.settings.alerts[arg] = false
            else
                db.settings.alerts[arg] = true
            end
            f:print(string.format("Alerts for %s: %s", arg, db.settings.alerts[arg] and "ON" or "OFF"))
        else
            f:print("Invalid input. Valid arguments for |*alerts*| are: %s", f:GetValidString(db.settings.alerts))
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["alpha"] = function(alpha, barID)
        alpha = tonumber(alpha)

        if not alpha or alpha < 0  or alpha > 1 then
            f:print("Alpha must be between 0 and 1.")
        else
            f:TransformBar("alpha", alpha, "SetAlpha", barID)
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["bar"] = function(arg, barID)
        if arg == "add" then
            f:AddBar()
        elseif arg == "remove" then
            if UnitAffectingCombat("player") then
                f:print("Can't complete this action during combat. Try again when combat ends.")
                return
            end

            barIDnum = tonumber(barID)

            if profile.bars[barIDnum] then
                local dialog = StaticPopup_Show("FARMITPLUS_CONFIRM_REMOVE_BAR", barID)
                if dialog then
                    dialog.data = barIDnum
                end
            else
                f:print("Invalid bar number. Available bars are: %s", f:GetValidString(profile.bars))
            end
        elseif arg == "list" then
            f:print("Available bars are: %s", f:GetValidString(profile.bars))
        else
            f:print("Valid arguments for |cff00ccffbar|r are: %s", f:GetValidString(f.ValidArguments["bar"], true))
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["clear"] = function(arg)
        if arg == "cache" then
            f:ClearCache()
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["grow"] = function(barID, orientation)
        if UnitAffectingCombat("player") then
            f:print("Can't complete this action during combat. Try again when combat ends.")
            return
        end

        barID = tonumber(barID)

        if not f.bars[barID] then
            f:print("Invalid bar ID. Available bars: %s", f:GetValidString(profile.bars))
        elseif not f.ValidArguments["grow"][strupper(orientation or "")] then
            f:print("Valid arguments for |cff00ccffgrow %s|r are: %s", barID, f:GetValidString(f.ValidArguments["grow"]))
        else
            profile.bars[barID].orientation = f.ValidArguments["grow"][strupper(orientation)]
            f:LoadButtons(f.bars[barID])
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["help"] = function()
        f:LoadTab("Help")
    end,


    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["hide"] = function(barID)
        if UnitAffectingCombat("player") then
            f:print("Can't complete this action during combat. Try again when combat ends.")
            return
        end

        f:TransformBar("hidden", true, "Hide", barID)
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["lock"] = function(barID)
        f:TransformBar("movable", false, "SetMovable", barID, L["%s: position locked."])
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["rebuild"] = function(arg)
        if arg and not tContains(f.ValidArguments["rebuild"], strlower(arg)) then
            f:print("Valid arguments for |cff00ccffrebuild|r are: %s", f:GetValidString(f.ValidArguments["rebuild"], true))
            return false
        end

        arg = arg or "all"
  
        local dialog = StaticPopup_Show("FARMITPLUS_CONFIRM_REBUILD", arg)
        if dialog then
            dialog.data = arg
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["reset"] = function()
        local dialog = StaticPopup_Show("FARMITPLUS_CONFIRM_RESET", arg)
        if dialog then
            dialog.data = arg
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["scale"] = function(scale, barID)
        if UnitAffectingCombat("player") then
            f:print("Can't complete this action during combat. Try again when combat ends.")
            return
        end

        scale = tonumber(scale) or 0

        if scale < 0.25 or scale > 5 then
            f:print("Scale must be between 0.25 and 5.")
        else
            f:TransformBar("scale", scale, "SetScale", barID)
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["size"] = function(barID, size)
        if UnitAffectingCombat("player") then
            f:print("Can't complete this action during combat. Try again when combat ends.")
            return
        end

        barID = tonumber(barID)
        size = tonumber(size)

        if not f.bars[barID] then
            f:print("Invalid bar ID. Available bars: %s", f:GetValidString(profile.bars))
        elseif not size or size < 0 or size > f.maxButtons then
            f:print(string.format("Size must be between 0 and %d.", f.maxButtons))
        else
            f:SetBarSize(barID, size)
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["show"] = function(barID)
        if UnitAffectingCombat("player") then
            f:print("Can't complete this action during combat. Try again when combat ends.")
            return
        end

        f:TransformBar("hidden", false, "Show", barID)
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["style"] = function(style)
        if style and tContains(f.ValidArguments["style"], strlower(style)) then
            if db.settings.style ~= strlower(style) then
                db.settings.style = strlower(style)

                StaticPopup_Show("FARMITPLUS_CONFIRM_RELOAD")
            else
                f:print("Style \"%s\" is already loaded. Available styles are: %s", style, f:GetValidString(f.ValidArguments["style"], true))
            end
        elseif not style then
            f:print("Please specify a style to load. Available styles are: %s", f:GetValidString(f.ValidArguments["style"], true))
        else
            f:print("Style \"%s\" could not be found. Available styles are: %s", style, f:GetValidString(f.ValidArguments["style"], true))
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["tooltip"] = function(frame)
        if frame and tContains(f.ValidArguments["tooltip"], strlower(frame)) then
            if db.settings.tooltip[frame] then
                db.settings.tooltip[frame] = false
            else
                db.settings.tooltip[frame] = true
            end
            f:print("Tooltips for %s: %s", strlower(frame), db.settings.tooltip[frame] and "ENABLED" or "DISABLED")
        else
            f:print("Invalid input. Valid arguments for |cff00ccfftooltip|r are: %s", f:GetValidString(db.settings.tooltip))
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["tpl"] = function(arg, ...)
        if arg == "save" then
            local args = {...}
            barID = tonumber(args[1])
            tremove(args, 1)
            local templateName = ""
            for k, v in pairs(args) do
                templateName = templateName == "" and v or string.format("%s %s", templateName, v)
            end

            if not f.bars[barID] then
                f:print("Invalid bar ID. Available bars: %s", f:GetValidString(profile.bars))
            elseif not templateName then
                f:print("Please provide a template name when saving an item bar: |cff00ccff/farmingbar tpl save %d MyTemplate|r", barID)
            else
                f:SaveTemplate(templateName, barID)
            end
        elseif arg == "load" then
            if UnitAffectingCombat("player") then
                f:print("Can't complete this action during combat. Try again when combat ends.")
                return
            end
            
            local args = {...}
            barID = tonumber(args[1])
            tremove(args, 1)
            local templateName = ""
            for k, v in pairs(args) do
                templateName = templateName == "" and v or string.format("%s %s", templateName, v)
            end

            if not f.bars[barID] then
                f:print("Invalid bar ID. Available bars: %s", f:GetValidString(profile.bars))
            elseif not templateName then
                f:print("Please specify a template to load.")
            elseif not db.templates[strupper(templateName)] then
                f:print("Template \"%s\" not found. Available templates: %s", templateName, f:GetValidString(db.templates))
            else
                f:LoadTemplate("U:" .. templateName, barID)
            end
        elseif arg == "delete" then
            local args = {...}
            local templateName = ""
            for k, v in pairs(args) do
                templateName = templateName == "" and v or string.format("%s %s", templateName, v)
            end


            if not templateName then
                f:print("Please specify a template to delete.")
            elseif not db.templates[strupper(templateName)] then
                f:print("Template \"%s\" not found. Available templates: %s", templateName, f:GetValidString(db.templates))
            else
                f:DeleteTemplate(strupper(templateName))
            end
        elseif arg == "list" then
            f:print("Available templates: %s", f:GetValidString(db.templates))
        else
            f:print("Valid arguments for |cff00ccfftpl|r are: %s", f:GetValidString(f.ValidArguments["tpl"], true))
        end
    end,

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    ["unlock"] = function(barID)
        f:TransformBar("movable", true, "SetMovable", barID, L["%s: position unlocked."])
    end,
}