local addon, ns = ...

local f = ns.f
local db = ns.db
local profile = ns.profile
local L = ns.L

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:CreateButton(parent, style, attributes, frameName, ...)
    local button = CreateFrame("Button", string.format("$parent$sButton", frameName), parent, "UIPanelButtonTemplate")
    button:SetText(attributes.text)
    if attributes.size[1] == "text" then
        attributes.size[1] = button:GetFontString():GetWidth() + 20
    end
    button:SetSize(unpack(attributes.size))

    button:SetScript("OnClick", function(...)
        attributes.func(...)
    end)

    if attributes.onUpdate then
        attributes.onUpdate(button, ...)

        button:SetScript("OnShow", function(...)
            attributes.onUpdate(button, ...)
        end)
    end

    style = style.elements
    if style.button then
        style = style.button

        f:SetButtonTexture(button, style, "Button")

        if style.Normal.pushedOffset then
            button:SetPushedTextOffset(f:unpack(style.Normal.pushedOffset))
        end

        if style.Normal.fonts then
            button:SetNormalFontObject(style.Normal.fonts.normal)
            button:SetHighlightFontObject(style.Normal.fonts.highlight)
        end
    end

    if attributes.textureStyle then
        f:SetButtonTexture(button, attributes.textureStyle, "Button")
    end

    return button
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:CreateCheckButton(parent, style, attributes, frameName, ...)
    local checkButton = CreateFrame("Button", string.format("$parent$sCheckButton", frameName), parent)
    checkButton:SetText(attributes.text)
    checkButton:SetPushedTextOffset(0, 0)
    checkButton:SetNormalFontObject(style.fonts.normal)

    style = style.elements.checkButton
    checkButton.texture = CreateFrame("Frame", nil, checkButton)
    checkButton.texture:SetSize(style.size or 14, style.size or 14)
    checkButton.texture:SetPoint("LEFT", 5, 0)

    if checkButton.checked then
        f:SetFrameTexture(checkButton.texture, style.Checked)
    else
        f:SetFrameTexture(checkButton.texture, style.Normal)
    end

    checkButton:SetSize(checkButton:GetFontString():GetWidth() + 20 + (style.size or 14), 20)
    hooksecurefunc(checkButton, "SetText", function(self)
        self:SetSize(self:GetFontString():GetWidth() + 20 + (style.size or 14), 20)
    end)
    checkButton:GetFontString():SetPoint("LEFT", checkButton.texture, "RIGHT", 5, 0)

    checkButton.SetChecked = function(self, setChecked)
        if setChecked then
            self.checked = true
            f:SetFrameTexture(self.texture, style.Checked)
        else
            self.checked = false
            f:SetFrameTexture(self.texture, style.Normal)
        end
    end

    checkButton.GetChecked = function(self)
        return self.checked
    end

    checkButton:SetScript("OnClick", function(self)
        self:SetChecked(not self:GetChecked())
        attributes.func(self)
    end)

    checkButton:SetScript("OnEnter", function(self)
        if style.Highlight then
            f:SetFrameTexture(self.texture, style.Highlight)
        end
    end)

    checkButton:SetScript("OnLeave", function(self)
        if style.Highlight then
            if self.checked then
                f:SetFrameTexture(self.texture, style.Checked)
            else
                f:SetFrameTexture(self.texture, style.Normal)
            end
        end
    end)

    if attributes.onUpdate then
        checkButton:SetScript("OnShow", function(self)
            attributes.onUpdate(self)
        end)

        attributes.onUpdate(checkButton)
    end

    return checkButton
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:CreateDropDown(parent, style, attributes, frameName, ...)
    local dropDown = CreateFrame("Button", string.format("$parent%sDropDown", frameName), parent)
    dropDown.Button = CreateFrame("Button", nil, dropDown)

    dropDown:SetSize(attributes.width or 175, attributes.height or 25)
    dropDown.Button:SetSize(dropDown:GetHeight() * .75, dropDown:GetHeight() * .75)
    dropDown.Button:SetPoint("RIGHT", -5, 0)

    dropDown:HookScript("OnEnable", function(self)
        self.Button:Enable()
    end)
    dropDown:HookScript("OnDisable", function(self)
        self.Button:Disable()
    end)

    attributes.defaultText = attributes.defaultText or " "
    dropDown:SetPushedTextOffset(0, 0)
    dropDown:SetNormalFontObject(style.fonts.normal)
    dropDown:SetDisabledFontObject(style.fonts.disable)
    dropDown:SetText(L[attributes.defaultText])
    dropDown:GetFontString():SetJustifyH(attributes.justify or "RIGHT")
    dropDown:GetFontString():SetWordWrap(false)
    dropDown:GetFontString():SetPoint("LEFT", dropDown, "LEFT", 5, 0)
    dropDown:GetFontString():SetPoint("RIGHT", dropDown.Button, "LEFT", -5, 0)

    dropDown:SetScript("OnClick", function(self, ...)
        if not self.Menu:IsVisible() then
            f:CloseDropDownMenus()
            self.Menu:Show()
        else
            f:CloseDropDownMenus()
        end
    end)
    dropDown.Button:SetScript("OnClick", function() dropDown:Click() end)

    if attributes.onUpdate then
        dropDown:SetScript("OnShow", function(self)
            attributes.onUpdate(self)
        end)

        attributes.onUpdate(dropDown)
    end

    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    dropDown.Menu = CreateFrame("Frame", dropDown:GetName() .. "Menu", UIParent)
    dropDown.Menu:Hide()
    dropDown.Menu:SetFrameStrata("TOOLTIP")
    dropDown.Menu:SetPoint("TOPLEFT", dropDown, "BOTTOMLEFT", -1, -5)
    dropDown.Menu:SetPoint("RIGHT", dropDown, "RIGHT", 1, -5)
    dropDown.Menu:SetSize(dropDown:GetWidth(), 150)

    dropDown.Menu.ScrollFrame = f:CreateScrollFrame("Menu", dropDown.Menu)
    dropDown.Menu.ScrollFrame:SetAllPoints(dropDown.Menu)
    dropDown.Menu.ScrollFrame.ScrollContent = f:CreateScrollContent(dropDown.Menu.ScrollFrame, 1, true)

    local lines = {}
    dropDown.Menu:SetScript("OnShow", function(self, ...)
        local info = {}
        attributes.onInitialize(info)

        for k, v in pairs(lines) do
            v:ClearAllPoints()
            v:Hide()
        end

        local height = 0
        for k, v in f:pairsByKeys(info) do
            height = height + 25
            if height > 300 then
                self.ScrollFrame.ScrollBar:Show()
                self.ScrollFrame:SetPoint("RIGHT", -(self.ScrollFrame.ScrollBar:GetWidth() + 5), 0)
                self:SetHeight(300)
            else
                self.ScrollFrame.ScrollBar:Hide()
                self.ScrollFrame:SetPoint("RIGHT", -5, 0)
                self:SetHeight(height + 5)
            end
            self.ScrollFrame:SetHeight(self:GetHeight())

            local button = CreateFrame("CheckButton", nil, self.ScrollFrame.ScrollContent)
            button:SetPoint("TOPLEFT", k > 1 and lines[k - 1] or self.ScrollFrame.ScrollContent, "BOTTOMLEFT", k > 1 and 0 or 5, -5)
            button:SetSize(self.ScrollFrame:GetWidth(), 20)
            button:SetHighlightTexture("Interface\\BUTTONS\\UI-Listbox-Highlight2")

            if not v.notCheckable and not v.isTitle then
                button.texture = CreateFrame("Frame", nil, button)
                button.texture:SetSize(style.elements.listButton.size or 14, style.elements.listButton.size or 14)
                button.texture:SetPoint("LEFT", 5, 0)
                if v.checked then
                    f:SetFrameTexture(button.texture, style.elements.listButton.Checked)
                else
                    f:SetFrameTexture(button.texture, style.elements.listButton.Normal)
                end
            end

            button:SetScript("OnClick", function(self)
                attributes.onSelect(dropDown, v.arg1)
                self:GetParent():GetParent():GetParent():Hide()
            end)

            local label = button:CreateFontString(nil, "OVERLAY", v.isTitle and style.fonts.header or style.fonts.normal)
            label:SetJustifyH("LEFT")
            label:SetText(v.text)
            label:SetPoint("LEFT", not button.texture and button or button.texture, not button.texture and "LEFT" or "RIGHT", 5, 0)
            label:SetPoint("RIGHT", self.ScrollFrame, "RIGHT", -5, 0)
            label:SetWordWrap(false)

            if v.isTitle then
                button:Disable()
            end

            lines[k] = button
        end
    end)


    hooksecurefunc(f, "CloseDropDownMenus", function()
        dropDown.Menu:Hide()
    end)
    hooksecurefunc("ToggleDropDownMenu", function()
        dropDown.Menu:Hide()
    end)


    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    if style.elements.dropDown then
        f:SetButtonTexture(dropDown, style.elements.dropDown, "Button")
    end
    if style.elements.dropDownMenu then
        f:SetFrameTexture(dropDown.Menu, style.elements.dropDownMenu)
    end
    if style.elements.dropDownButton then
        f:SetButtonTexture(dropDown.Button, style.elements.dropDownButton, "Button")
    end

    if style.elements.dropDownScrollFrame then
        dropDown.Menu.ScrollFrame.ScrollBar:SetBackdrop(style.elements.dropDownScrollFrame.track)
        dropDown.Menu.ScrollFrame.ScrollBar:SetBackdropColor(f:unpack(style.elements.dropDownScrollFrame.track.bgColor))
        dropDown.Menu.ScrollFrame.ScrollBar:SetBackdropBorderColor(0, 0, 0, 0)

        local upButton = dropDown.Menu.ScrollFrame.ScrollBar.ScrollUpButton
        local downButton = dropDown.Menu.ScrollFrame.ScrollBar.ScrollDownButton

        f:SetButtonTexture(upButton, style.elements.dropDownScrollFrame.upButton, "Button", true)
        f:SetButtonTexture(downButton, style.elements.dropDownScrollFrame.downButton, "Button", true)

        local thumb = dropDown.Menu.ScrollFrame.ScrollBar.ThumbTexture
        thumb:SetWidth(dropDown.Menu.ScrollFrame.ScrollBar:GetWidth() -2)
        thumb:SetTexture(style.elements.dropDownScrollFrame.thumb.bgFile)
        thumb:SetColorTexture(f:unpack(style.elements.dropDownScrollFrame.thumb.bgColor))
    end

    return dropDown
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:CreateEditBox(parent, style, attributes, frameName, ...)
    local editBox = CreateFrame("EditBox", string.format("$parent%sEditBox", frameName), parent, "InputBoxTemplate")
    editBox:SetSize(attributes.width or 175, attributes.height or 25)
    editBox:SetAutoFocus(attributes.autoFocus)
    editBox:SetFontObject(style.fonts.normal)
    editBox:SetMaxLetters(attributes.maxLetters or 9)
    editBox:SetJustifyH(attributes.justify or "LEFT")

    editBox:SetScript("OnEscapePressed", function(self) self:ClearFocus() end)
    editBox:SetScript("OnTextChanged", function(self, ...)
        if attributes.validateText then
            local text = string.gsub(self:GetText(), attributes.validateText, "")
            self:SetText(text)
        end

        if attributes.showButton then
            if self:GetText() ~= "" then
                self.Button:Show()
            else
                self.Button:Hide()
            end
        end
    end)

    editBox:SetScript("OnEnterPressed", function(self, ...)
        attributes.onEnterPressed(self, ...)
        self:ClearFocus()
    end)
    if attributes.onFocusGained then
        editBox:SetScript("OnEditFocusGained", function(self)
            attributes.onFocusGained(self)
        end)
    end
    if attributes.onUpdate then
        editBox:SetScript("OnShow", function(self)
            attributes.onUpdate(self)
        end)
        attributes.onUpdate(editBox)
    end

    editBox.Button = f:CreateButton(editBox, style, {
        text = "Okay",
        size = {35, editBox:GetHeight() - 10},
        func = function()
            attributes.onEnterPressed(editBox)
        end
    })

    editBox.Button:SetPoint("RIGHT", editBox, "RIGHT", -5, 0)
    editBox.Button:Hide()

    if attributes.insets then
        editBox:SetTextInsets(f:unpack(attributes.insets))
    else
        editBox:SetTextInsets(5, 5 + (attributes.showButton and editBox.Button:GetWidth() or 0), 1, 1)
    end

    style = style.elements
    if style.editBox then
        local style = style.editBox

        editBox.Left:SetTexture()
        editBox.Middle:SetTexture()
        editBox.Right:SetTexture()

        local textureFrame = CreateFrame("Frame", nil, parent)
        f:SetFrameTexture(textureFrame, style)
        textureFrame:SetAllPoints(editBox)
    end

    return editBox
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:CreateFontBlock(parent, style, attributes, _, isScrollContent)
    local fontString = parent:CreateFontString(nil, "OVERLAY", attributes.header and style.fonts.header or style.fonts.normal)
    if not attributes.sizeToText then
        fontString:SetWidth((isScrollContent and parent:GetParent():GetWidth() or parent:GetWidth()) - 20)
        fontString:SetJustifyH(attributes.justify or "LEFT")
        fontString:SetWordWrap(true)
    end

    fontString:SetText(f:ParseText(attributes.text))

    return fontString
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:CreateScrollContent(scrollFrame, frameID, setChild)
    local scrollContent = CreateFrame("Frame", string.format("$parentScrollContent%d", frameID), scrollFrame)
    scrollContent:SetSize(1, 1)
    scrollContent:SetAllPoints(scrollFrame)

    if setChild then
        scrollFrame:SetScrollChild(scrollContent)
    end

    return scrollContent
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:CreateScrollFrame(frameName, parent)
    local scrollFrame = CreateFrame("ScrollFrame", string.format("$parent%sScrollFrame", frameName), parent, "UIPanelScrollFrameTemplate")

    local style = f:GetStyle(db.settings.style).elements
    if style.scrollFrame then
        style = style.scrollFrame

        scrollFrame.ScrollBar:SetBackdrop(style.track)
        scrollFrame.ScrollBar:SetBackdropColor(f:unpack(style.track.bgColor))
        scrollFrame.ScrollBar:SetBackdropBorderColor(0, 0, 0, 0)

        local upButton = scrollFrame.ScrollBar.ScrollUpButton
        local downButton = scrollFrame.ScrollBar.ScrollDownButton

        f:SetButtonTexture(upButton, style.upButton, "Button", true)
        f:SetButtonTexture(downButton, style.downButton, "Button", true)

        local thumb = scrollFrame.ScrollBar.ThumbTexture
        thumb:SetWidth(scrollFrame.ScrollBar:GetWidth() -2)
        thumb:SetTexture(style.thumb.bgFile)
        thumb:SetColorTexture(f:unpack(style.thumb.bgColor))
    end

    scrollFrame.ScrollBar:EnableMouseWheel(true)
    scrollFrame.ScrollBar:SetScript("OnMouseWheel", function(self, direction)
        ScrollFrameTemplate_OnMouseWheel(scrollFrame, direction)
    end)

    return scrollFrame
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:CreateSlider(parent, style, attributes, frameName, ...)
    local slider = CreateFrame("Slider", string.format("$parent%sSlider", frameName), parent, "OptionsSliderTemplate")
    slider:SetSize(attributes.width or 200, attributes.height or 20)
    slider:SetOrientation(attributes.orientation or "HORIZONTAL")
    slider.Low:SetText(attributes.minValue)
    slider.High:SetText(attributes.maxValue)
    slider:SetMinMaxValues(attributes.minValue, attributes.maxValue)
    slider:SetValueStep(attributes.stepValue or 1)
    slider:SetObeyStepOnDrag(true)

    if attributes.onUpdate then
        slider:SetScript("OnShow", function(self)
            attributes.onUpdate(self)
        end)
        attributes.onUpdate(slider)
    end

    slider.Text:SetText(attributes.defaultText)
    slider.Text:SetFontObject(style.fonts.header)

    local editBox = f:CreateEditBox(parent, style, {
        validateText = "[^%d%.]", -- numbers/decimal only
        width = slider:GetWidth() / 3,
        justify = "CENTER",
        maxLetters = attributes.maxLetters,
        onEnterPressed = function(self, ...)
            local min, max = slider:GetMinMaxValues()
            local value = self:GetNumber()

            if value < min then
                value = min
            elseif value > max then
                value = max
            end

            value = f:round(value, attributes.round)

            attributes.onEnterPressed(value, slider)
            slider:SetValue(value)
            self:SetText(value)
        end,
        onFocusGained = function(self, ...)
            self:HighlightText()
            f:CloseDropDownMenus()
        end,
    }, frameName)
    slider.editBox = editBox
    editBox:SetPoint("TOP", slider, "BOTTOM", 0, -5)
    editBox:SetText(slider:GetValue())
    editBox:SetEnabled(slider:IsEnabled())

    slider:SetScript("OnValueChanged", function(self, value)
        value = f:round(value, attributes.round)
        if self.editBox:GetNumber() ~= value then
            self.editBox:SetText(value)
        end
        attributes.onValueChanged(self, value)
    end)

    slider:SetScript("OnEnable", function(self)
        self.editBox:Enable()
        self.Thumb:Show()
        self.Low:Show()
        self.High:Show()
        self.editBox:SetText(f:round(self:GetValue(), attributes.round))
    end)

    slider:SetScript("OnDisable", function(self)
        self.editBox:Disable()
        self.Thumb:Hide()
        self.Low:Hide()
        self.High:Hide()
        self.editBox:SetText("")
    end)

    slider.Low:SetPoint("TOPLEFT", slider, "BOTTOMLEFT", 0, -2)
    slider.High:SetPoint("TOPRIGHT", slider, "BOTTOMRIGHT", 0, -2)

    style = style.elements
    if style.slider then
        if style.slider.height then
            slider:SetHeight(style.slider.height)
        end

        slider:SetBackdrop({
            bgFile = style.slider.type == "texture" and style.slider.texture or "Interface\\BUTTONS\\WHITE8X8",
            edgeFile = style.slider.borderTexture and style.slider.borderTexture or "Interface\\BUTTONS\\WHITE8X8",
            edgeSize = style.slider.border,
        })

        if style.slider.type == "color" then
            slider:SetBackdropColor(f:unpack(style.slider.texture))
        end

        if style.slider.borderColor then
            slider:SetBackdropBorderColor(f:unpack(style.slider.borderColor))
        end

        if style.sliderThumb.type == "color" then
            slider.Thumb:SetColorTexture(f:unpack(style.sliderThumb.texture))
        else
            slider.Thumb:SetTexture(style.sliderThumb.texture)
        end
        slider.Thumb:SetSize(style.sliderThumb.width, slider:GetHeight() -2)
    end

    return slider
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

local loaded
function f:LoadOptions()
    if not loaded then
        local style = f:GetStyle(db.settings.style)
        f:SetFrameTexture(f, style.optionsFrame.main)
        f:SetMovable(true)
        f:SetSize(600, 450)
        f:SetPoint("CENTER", 0, 0)

        f:EnableMouse(true)
        f:SetScript("OnMouseUp", function(self, button, ...)
            if button == "LeftButton" then
                f:CloseDropDownMenus()
            end
        end)

        local closeButton = CreateFrame("Button", "$parentCloseButton", f, "UIPanelCloseButton")
        closeButton:SetPoint("TOPRIGHT", 4, 4)

        if style.optionsFrame.closeButton then
            f:SetButtonTexture(closeButton)

            closeButton:SetNormalFontObject(style.optionsFrame.closeButton.normal)
            closeButton:SetHighlightFontObject(style.optionsFrame.closeButton.highlight)
            closeButton:SetPushedTextOffset(0, 0)
            closeButton:SetText("X")
        end

        local titleFrame = CreateFrame("Frame", "$parentTitleFrame", f)
        titleFrame:SetPoint("TOPLEFT", 0, 0)
        titleFrame:SetPoint("BOTTOMRIGHT", closeButton, "BOTTOMLEFT", 0, 0)

        titleFrame:EnableMouse(true)
        titleFrame:RegisterForDrag("LeftButton")
        titleFrame:SetScript("OnDragStart", function(self, ...) self:GetParent():StartMoving() end)
        titleFrame:SetScript("OnDragStop", function(self) self:GetParent():StopMovingOrSizing() end)

        titleFrame:SetScript("OnMouseUp", function(self, button, ...)
            if button == "LeftButton" then
                f:CloseDropDownMenus()
            end
        end)

        titleFrame.title = titleFrame:CreateFontString(nil, "OVERLAY", style.fonts.header)
        titleFrame.title:SetPoint("LEFT", 10, 2)
        titleFrame.title:SetText(addon)

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Frame textures

        local cornerTexture = f:CreateTexture(nil, "BORDER")
        cornerTexture:SetTexture(style.optionsFrame.cornerTexture)
        cornerTexture:SetSize(32, 32)
        cornerTexture:SetPoint("TOPRIGHT", 0, 0)

        local dividerTexture = f:CreateTexture(nil, "BORDER")
        dividerTexture:SetTexture(style.optionsFrame.divider.bgFile)
        dividerTexture:SetSize(f:GetWidth(), style.optionsFrame.divider.height)
        dividerTexture:SetPoint("BOTTOMLEFT", titleFrame, "BOTTOMLEFT", 0, 2)

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Tabs

        f.tabs = {
            {title = "About"},
            {title = "Commands"},
            {title = "Config"},
            {title = "Settings"},
            {title = "Help"},
        }

        for k, v in f:pairsByKeys(f.tabs) do
            local button = CreateFrame("Button", string.format("$parent%sTabButton", v.title), f)
            button:SetText(L[v.title])
            button:SetNormalFontObject("GameFontNormal")
            button:SetHighlightFontObject("GameFontHighlight")
            button:GetFontString():SetPoint("LEFT", 5, 0)
            button:SetSize(button:GetFontString():GetWidth() + 10, 30)
            if k == 1 then
                button:SetPoint("TOPLEFT", titleFrame, "BOTTOMLEFT", 10, 0)
            else
                button:SetPoint("LEFT", f.tabs[k - 1].button, "RIGHT", 10, 0)
            end

            f.tabs[k].button = button
        end

        -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Main frame

        local insetFrame = f:CreateScrollFrame("MainFrame", f)
        insetFrame:SetPoint("TOPLEFT", f.tabs[1].button, "BOTTOMLEFT", 0, 0)
        insetFrame:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -30, 10)

        f:SetFrameTexture(insetFrame, style.optionsFrame.inset)

        for i = 1, f:count(f.tabs) do
            f.tabs[i].scrollContent = f:CreateScrollContent(insetFrame, i)

            f.tabs[i].button:SetScript("OnClick", function(self, ...)
                insetFrame:SetScrollChild(f.tabs[i].scrollContent)
                f[string.format("Load%sTab", f.tabs[i].title)](self, i)

                for k, v in pairs(f.tabs) do
                    if k == i then
                        v.button:SetNormalFontObject("GameFontHighlight")
                        v.scrollContent:Show()
                    else
                        v.button:SetNormalFontObject("GameFontNormal")
                        v.scrollContent:Hide()
                    end
                end
                f:CloseDropDownMenus()
            end)
        end

        loaded = true
    end

    f:Show()
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:LoadTab(tabName)
    tabName = tabName or "About"
    f:LoadOptions()

    for k, v in pairs(f.tabs) do
        if v.title == tabName then
            v.button:Click()
        end
    end
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

function f:RenderElements(parent, anchor, elementsTable, isScrollContent)
    local style = f:GetStyle(db.settings.style)
    local lines = {}

    local i = 1
    local offset = 1
    for k, v in f:pairsByKeys(elementsTable) do
        lines[i] = lines[i] or f["Create" .. v.type](f, parent, style, v.attributes, i, isScrollContent)

        if i > 1 then
            anchor = lines[i - (v.group and 1 or offset)]
            if v.group and anchor.label then
                anchor = anchor.label
            end
        end

        lines[i]:SetPoint(
            v.group and "LEFT" or "TOPLEFT",
            anchor,
            v.group and "RIGHT" or "BOTTOMLEFT",
            (v.offsetX or 0) + (v.group and 10 or 0),
            (v.offsetY or 0) + (v.group and 0 or (v.attributes.header and -10 or -5))
        )

        if v.type ~= "DropDown" then
            if v.type == "EditBox" then
                lines[i]:HookScript("OnEditFocusGained", function()
                    f:CloseDropDownMenus()
                end)
            elseif v.type == "Slider" then
                lines[i]:HookScript("OnValueChanged", function()
                    f:CloseDropDownMenus()
                end)
            elseif v.type ~= "Slider" and lines[i].GetScript and lines[i]:GetScript("OnClick") then
                lines[i]:HookScript("OnClick", function()
                    f:CloseDropDownMenus()
                end)
            end
        end

        if v.group then
            offset = offset + 1
        else
            offset = 1
        end

        i = i + 1
    end

    if isScrollContent then
        lines[i] = parent:CreateFontString(nil, "OVERLAY")
        lines[i]:SetPoint("TOPLEFT", lines[i - 1], "BOTTOMLEFT", 0, -5)
    end
end